@@ -2,7 +2,7 @@
    and variance 1 and round it to the precision of rop according to the given
    rounding mode.
 
-Copyright 2013-2022 Free Software Foundation, Inc.
+Copyright 2013-2025 Free Software Foundation, Inc.
 Contributed by Charles Karney <charles@karney.com>, SRI International.
 
 This file is part of the GNU MPFR Library.
@@ -18,9 +18,8 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
 /*
  * Sampling from the normal distribution with zero mean and unit variance.
@@ -31,6 +30,11 @@ https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
  *   https://dx.doi.org/10.1145/2710016
  *   https://arxiv.org/abs/1303.6257
  *
+ * Note: the algorithm implemented here has been improved in
+ * Du, Fan and Wei in "An improved exact sampling algorithm for the standard
+ * normal distribution", Computational Statistics, 2021,
+ * https://doi.org/10.1007/s00180-021-01136-w
+ *
  * The implementation here closely follows the C++ one given in the paper
  * above.  However, here, C is simplified by using gmp_urandomm_ui; the initial
  * rejection step in H just tests the leading bit of p; and the assignment of
