@@ -1,7 +1,7 @@
 /* mpfr_set_flt -- convert a machine single precision float to mpfr_t
 
-Copyright 2009-2015 Free Software Foundation, Inc.
-Contributed by the AriC and Caramel projects, INRIA.
+Copyright 2009-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,9 +16,8 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
 #include "mpfr-impl.h"
 
@@ -26,9 +25,9 @@ int
 mpfr_set_flt (mpfr_ptr r, float f, mpfr_rnd_t rnd_mode)
 {
   /* we convert f to double precision and use mpfr_set_d;
-     NaN and infinities should be preserved, and all single precision
-     numbers are exactly representable in the double format, thus the
-     conversion is always exact */
+     NaN and infinities should be preserved (except the sign bit for NaN),
+     and all single precision numbers are exactly representable in the double
+     format, thus the conversion is always exact */
   return mpfr_set_d (r, (double) f, rnd_mode);
 }
 
