@@ -1,7 +1,7 @@
 /* Test file for mpfr_add and mpfr_sub.
 
-Copyright 1999-2022 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 1999-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,9 +16,8 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
 #define N 30000
 
@@ -748,7 +747,7 @@ check_1111 (void)
         mpfr_set (b, one, MPFR_RNDN);
       tc = 1 + (randlimb () % (prec_c - 1));
       mpfr_div_2ui (c, one, tc, MPFR_RNDN);
-      sc = randlimb () % 2;
+      sc = RAND_BOOL ();
       if (sc)
         mpfr_neg (c, c, MPFR_RNDN);
       test_add (c, c, one, MPFR_RNDN);
@@ -821,7 +820,7 @@ check_1minuseps (void)
         mpfr_sub (b, c, b, MPFR_RNDN);  /* b = 1 - 2^(-prec_a) */
 
         for (ic = 0; ic < numberof (supp_b); ic++)
-          for (rnd_mode = 0; rnd_mode < MPFR_RND_MAX; rnd_mode++)
+          RND_LOOP (rnd_mode)
             {
               mpfr_t s;
               int inex_a, inex_s;
