@@ -1,7 +1,7 @@
 /* Test file for mpfr_add1sp.
 
-Copyright 2004-2019 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 2004-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,15 +16,21 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
 #include "mpfr-test.h"
 
 static void check_special (void);
 static void check_random (mpfr_prec_t p);
 
+static int
+mpfr_add_cf (mpfr_ptr a, mpfr_srcptr b, mpfr_srcptr c, mpfr_rnd_t r)
+{
+  mpfr_clear_flags ();  /* allows better checking */
+  return mpfr_add (a, b, c, r);
+}
+
 static void
 check_overflow (void)
 {
@@ -64,7 +70,7 @@ bug20171217 (void)
   mpfr_init2 (c, 137);
   mpfr_set_str_binary (b, "0.11111111111111111111111111111111111111111111111111111111111111111111000000000000000000000000000000000000000000000000000000000000000000000E-66");
   mpfr_set_str_binary (c, "0.11111111111111111111111111111111111111111111111111111111111111111000000000000000000000000000000000000000000000000000000000000000000110000E-2");
-  mpfr_add (a, b, c, MPFR_RNDN);
+  mpfr_add_cf (a, b, c, MPFR_RNDN);
   mpfr_set_str_binary (b, "0.10000000000000000000000000000000000000000000000000000000000000000100000000000000000000000000000000000000000000000000000000000000000001000E-1");
   MPFR_ASSERTN(mpfr_equal_p (a, b));
   mpfr_clear (a);
@@ -72,6 +78,47 @@ bug20171217 (void)
   mpfr_clear (c);
 }
 
+static void
+bug20190903 (void)
+{
+  mpfr_t a, b, c, d;
+  int inex;
+  mpfr_flags_t flags;
+
+  /* Bug in r13574, fixed in r13578.
+     Note: to reproduce the failure, GMP_NUMB_BITS == 64 is assumed. */
+  mpfr_inits2 (128, a, b, c, d, (mpfr_ptr) 0);
+  mpfr_set_str_binary (b, "0.11111111111111111100000000000000000001001111111101111111110000101001111100111110110010011001111110000000101001001001110110101110E0");
+  mpfr_set_str_binary (c, "0.10000001011101000010000111100111011110100000001001000010011011001000110100111111101100001101001101011101100100011000000101110111E-126");
+  mpfr_add_cf (a, b, c, MPFR_RNDN);
+  mpfr_set_str_binary (b, "0.11111111111111111100000000000000000001001111111101111111110000101001111100111110110010011001111110000000101001001001110110110000E0");
+  MPFR_ASSERTN (mpfr_equal_p (a, b));
+  mpfr_clears (a, b, c, d, (mpfr_ptr) 0);
+
+  /* Bug in r13574, fixed in r13586. */
+  /* Figure with GMP_NUMB_BITS = 4:
+       b = 1111 1000
+       c =      1000 0001
+  */
+  mpfr_inits2 (2 * GMP_NUMB_BITS, a, b, c, d, (mpfr_ptr) 0);
+  mpfr_set_ui_2exp (d, 1, 3 * GMP_NUMB_BITS, MPFR_RNDN);
+  mpfr_set_ui_2exp (c, 1, 2 * GMP_NUMB_BITS - 1, MPFR_RNDN);
+  mpfr_sub (b, d, c, MPFR_RNDN);
+  mpfr_add_ui (c, c, 1, MPFR_RNDN);
+  inex = mpfr_add_cf (a, b, c, MPFR_RNDN);
+  flags = __gmpfr_flags;
+  MPFR_ASSERTN (mpfr_equal_p (a, d));
+  MPFR_ASSERTN (inex < 0);
+  MPFR_ASSERTN (flags == MPFR_FLAGS_INEXACT);
+  inex = mpfr_add_cf (a, b, c, MPFR_RNDU);
+  flags = __gmpfr_flags;
+  mpfr_add_ui (d, d, 1, MPFR_RNDU);
+  MPFR_ASSERTN (mpfr_equal_p (a, d));
+  MPFR_ASSERTN (inex > 0);
+  MPFR_ASSERTN (flags == MPFR_FLAGS_INEXACT);
+  mpfr_clears (a, b, c, d, (mpfr_ptr) 0);
+}
+
 /* Check corner case b = 1, c = 2^(-p) for MPFR_PREC_MIN <= p <= pmax.
    With RNDN, result is 1, except for p=1, where it is 2. */
 static void
@@ -88,7 +135,7 @@ test_corner_1 (mpfr_prec_t pmax)
       mpfr_init2 (c, p);
       mpfr_set_ui (b, 1, MPFR_RNDN);
       mpfr_set_ui_2exp (c, 1, -p, MPFR_RNDN);
-      inex = mpfr_add (a, b, c, MPFR_RNDN);
+      inex = mpfr_add_cf (a, b, c, MPFR_RNDN);
       if (p == 1) /* special case, since 2^(p-1) is odd */
         {
           MPFR_ASSERTN(inex > 0);
@@ -111,19 +158,27 @@ coverage (void)
   mpfr_t a, b, c;
   int inex;
   mpfr_exp_t emax;
+  mpfr_prec_t p;
 
-  /* coverage test in mpfr_add1sp1n: case round away, where add_one_ulp
+  mpfr_init (a);
+  mpfr_init (b);
+  mpfr_init (c);
+
+  /* coverage test in mpfr_add1sp: case round away, where add_one_ulp
      gives a carry, and the new exponent is below emax */
-  mpfr_init2 (a, GMP_NUMB_BITS);
-  mpfr_init2 (b, GMP_NUMB_BITS);
-  mpfr_init2 (c, GMP_NUMB_BITS);
-  mpfr_set_ui (b, 1, MPFR_RNDN);
-  mpfr_nextbelow (b); /* b = 1 - 2^(-p) */
-  mpfr_set_ui_2exp (c, 1, -GMP_NUMB_BITS-1, MPFR_RNDN);
-  /* c = 2^(-p-1) thus b+c = 1 - 2^(-p-1) should be rounded to 1 */
-  inex = mpfr_add (a, b, c, MPFR_RNDU);
-  MPFR_ASSERTN(inex > 0);
-  MPFR_ASSERTN(mpfr_cmp_ui (a, 1) == 0);
+  for (p = MPFR_PREC_MIN; p <= 3 * GMP_NUMB_BITS; p++)
+    {
+      mpfr_set_prec (a, p);
+      mpfr_set_prec (b, p);
+      mpfr_set_prec (c, p);
+      mpfr_set_ui (b, 1, MPFR_RNDN);
+      mpfr_nextbelow (b); /* b = 1 - 2^(-p) (including for p=1) */
+      mpfr_set_ui_2exp (c, 1, -p-1, MPFR_RNDN);
+      /* c = 2^(-p-1) thus b+c = 1 - 2^(-p-1) should be rounded to 1 */
+      inex = mpfr_add_cf (a, b, c, MPFR_RNDU);
+      MPFR_ASSERTN(inex > 0);
+      MPFR_ASSERTN(mpfr_cmp_ui (a, 1) == 0);
+    }
 
   /* coverage test in mpfr_add1sp2: case GMP_NUMB_BITS <= d < 2*GMP_NUMB_BITS
      and a1 = 0 */
@@ -135,7 +190,7 @@ coverage (void)
   mpfr_set_ui_2exp (c, 1, -GMP_NUMB_BITS-1, MPFR_RNDN);
   mpfr_nextbelow (c); /* c = 2^(1-p) - 2^(1-2p) */
   /* a = 1 + 2^(-p) - 2^(1-2p) should be rounded to 1 with RNDN */
-  inex = mpfr_add (a, b, c, MPFR_RNDN);
+  inex = mpfr_add_cf (a, b, c, MPFR_RNDN);
   MPFR_ASSERTN(inex < 0);
   MPFR_ASSERTN(mpfr_cmp_ui (a, 1) == 0);
 
@@ -148,7 +203,7 @@ coverage (void)
   mpfr_nextbelow (b); /* b = 1 - 2^(-p) */
   mpfr_set_ui_2exp (c, 1, -GMP_NUMB_BITS-2, MPFR_RNDN);
   /* c = 2^(-p-1) */
-  inex = mpfr_add (a, b, c, MPFR_RNDU);
+  inex = mpfr_add_cf (a, b, c, MPFR_RNDU);
   MPFR_ASSERTN(inex > 0);
   MPFR_ASSERTN(mpfr_cmp_ui (a, 1) == 0);
 
@@ -162,26 +217,26 @@ coverage (void)
   mpfr_set_ui_2exp (c, 1, -2*GMP_NUMB_BITS-1, MPFR_RNDN);
   mpfr_nextbelow (c); /* c = 2^(1-p) - 2^(1-2p) */
   /* a = 1 + 2^(-p) - 2^(1-2p) should be rounded to 1 with RNDN */
-  inex = mpfr_add (a, b, c, MPFR_RNDN);
+  inex = mpfr_add_cf (a, b, c, MPFR_RNDN);
   MPFR_ASSERTN(inex < 0);
   MPFR_ASSERTN(mpfr_cmp_ui (a, 1) == 0);
 
   /* coverage test in mpfr_add1sp3: case bx > emax */
   emax = mpfr_get_emax ();
-  mpfr_set_emax (1);
+  set_emax (1);
   mpfr_set_prec (a, 2 * GMP_NUMB_BITS + 1);
   mpfr_set_prec (b, 2 * GMP_NUMB_BITS + 1);
   mpfr_set_prec (c, 2 * GMP_NUMB_BITS + 1);
   mpfr_set_ui_2exp (b, 1, mpfr_get_emax () - 1, MPFR_RNDN);
   mpfr_nextbelow (b);
-  mpfr_mul_2exp (b, b, 1, MPFR_RNDN);
+  mpfr_mul_2ui (b, b, 1, MPFR_RNDN);
   /* now b is the largest number < +Inf */
-  mpfr_div_2exp (c, b, GMP_NUMB_BITS - 1, MPFR_RNDN);
+  mpfr_div_2ui (c, b, GMP_NUMB_BITS - 1, MPFR_RNDN);
   /* we are in the case d < GMP_NUMB_BITS of mpfr_add1sp3 */
-  inex = mpfr_add (a, b, b, MPFR_RNDU);
+  inex = mpfr_add_cf (a, b, b, MPFR_RNDU);
   MPFR_ASSERTN(inex > 0);
   MPFR_ASSERTN(mpfr_inf_p (a) && mpfr_sgn (a) > 0);
-  mpfr_set_emax (emax);
+  set_emax (emax);
 
   /* coverage test in mpfr_add1sp3: case round away, where add_one_ulp gives
      a carry, no overflow */
@@ -192,26 +247,26 @@ coverage (void)
   mpfr_nextbelow (b); /* b = 1 - 2^(-p) */
   mpfr_set_ui_2exp (c, 1, -2 * GMP_NUMB_BITS - 2, MPFR_RNDN);
   /* c = 2^(-p-1) */
-  inex = mpfr_add (a, b, c, MPFR_RNDU);
+  inex = mpfr_add_cf (a, b, c, MPFR_RNDU);
   MPFR_ASSERTN(inex > 0);
   MPFR_ASSERTN(mpfr_cmp_ui (a, 1) == 0);
 
   /* coverage test in mpfr_add1sp3: case round away, where add_one_ulp gives
      a carry, with overflow */
   emax = mpfr_get_emax ();
-  mpfr_set_emax (1);
+  set_emax (1);
   mpfr_set_prec (a, 2 * GMP_NUMB_BITS + 1);
   mpfr_set_prec (b, 2 * GMP_NUMB_BITS + 1);
   mpfr_set_prec (c, 2 * GMP_NUMB_BITS + 1);
   mpfr_set_ui_2exp (b, 1, mpfr_get_emax () - 1, MPFR_RNDN);
   mpfr_nextbelow (b);
-  mpfr_mul_2exp (b, b, 1, MPFR_RNDN);
+  mpfr_mul_2ui (b, b, 1, MPFR_RNDN);
   /* now b is the largest number < +Inf */
   mpfr_set_ui_2exp (c, 1, mpfr_get_emin () - 1, MPFR_RNDN);
-  inex = mpfr_add (a, b, c, MPFR_RNDU);
+  inex = mpfr_add_cf (a, b, c, MPFR_RNDU);
   MPFR_ASSERTN(inex > 0);
   MPFR_ASSERTN(mpfr_inf_p (a) && mpfr_sgn (a) > 0);
-  mpfr_set_emax (emax);
+  set_emax (emax);
 
   mpfr_clear (a);
   mpfr_clear (b);
@@ -222,15 +277,23 @@ int
 main (void)
 {
   mpfr_prec_t p;
+  int i;
 
   tests_start_mpfr ();
 
   coverage ();
   test_corner_1 (1024);
   bug20171217 ();
+  bug20190903 ();
   check_special ();
-  for(p = MPFR_PREC_MIN; p < 200 ; p++)
+  for (p = MPFR_PREC_MIN; p < 200; p++)
     check_random (p);
+  for (i = 0; i < 200; i++)
+    {
+      /* special precisions */
+      check_random (GMP_NUMB_BITS);
+      check_random (2 * GMP_NUMB_BITS);
+    }
   check_overflow ();
 
   tests_end_mpfr ();
@@ -287,9 +350,9 @@ check_random (mpfr_prec_t p)
       mpfr_urandom (c, RANDS, MPFR_RNDA);
       if (MPFR_IS_PURE_FP(b) && MPFR_IS_PURE_FP(c))
         {
-          if (randlimb () & 1)
+          if (RAND_BOOL ())
             mpfr_neg (b, b, MPFR_RNDN);
-          if (randlimb () & 1)
+          if (RAND_BOOL ())
             mpfr_neg (c, c, MPFR_RNDN);
           if (MPFR_GET_EXP(b) < MPFR_GET_EXP(c))
             {
@@ -303,7 +366,7 @@ check_random (mpfr_prec_t p)
               MPFR_ALIAS (bs, b, MPFR_SIGN (b), MPFR_EXP (b));
               MPFR_ALIAS (cs, c, MPFR_SIGN (c), MPFR_EXP (c));
             }
-          for (r = 0 ; r < MPFR_RND_MAX ; r++)
+          RND_LOOP (r)
             {
               mpfr_flags_t flags1, flags2;
 
@@ -340,7 +403,7 @@ check_special (void)
 
   mpfr_inits (a1, a2, b, c, (mpfr_ptr) 0);
 
-  for (r = 0 ; r < MPFR_RND_MAX ; r++)
+  RND_LOOP (r)
     {
       if (r == MPFR_RNDF)
         continue; /* inexact makes no sense, mpfr_add1 and mpfr_add1sp
