@@ -1,7 +1,7 @@
 /* talloc-cache  -- test file concerning memory allocation and cache
 
-Copyright 2016-2022 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 2016-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,9 +16,8 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
 #include <stdlib.h>
 
@@ -36,6 +35,8 @@ static void *
 my_alloc1 (size_t s)
 {
   void *p = malloc (s + A);
+  if (p == NULL)
+    abort ();
   MPFR_ASSERTN (v == 1);
   *(int *) p = 1;
   return (void *) ((char *) p + A);
@@ -63,6 +64,8 @@ static void *
 my_alloc2 (size_t s)
 {
   void *p = malloc (s + A);
+  if (p == NULL)
+    abort ();
   MPFR_ASSERTN (v == 2);
   *(int *) p = 2;
   return (void *) ((char *) p + A);
