@@ -1,7 +1,7 @@
-/* Test file for mpfr_atan.
+/* Test file for mpfr_atan and mpfr_atan2.
 
-Copyright 2001-2018 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 2001-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,9 +16,8 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
 #include "mpfr-test.h"
 
@@ -50,12 +49,12 @@ special (void)
     }
 
   /* atan(+Inf) = Pi/2 */
-  for (r = 0; r < MPFR_RND_MAX ; r++)
+  RND_LOOP (r)
     {
       mpfr_set_inf (x, 1);
       mpfr_atan (y, x, (mpfr_rnd_t) r);
       mpfr_const_pi (x, (mpfr_rnd_t) r);
-      mpfr_div_2exp (x, x, 1, (mpfr_rnd_t) r);
+      mpfr_div_2ui (x, x, 1, (mpfr_rnd_t) r);
       if (mpfr_cmp (x, y))
         {
           printf ("Error: mpfr_atan(+Inf), rnd=%s\n",
@@ -65,13 +64,13 @@ special (void)
     }
 
   /* atan(-Inf) = - Pi/2 */
-  for (r = 0; r < MPFR_RND_MAX ; r++)
+  RND_LOOP (r)
     {
       mpfr_set_inf (x, -1);
       mpfr_atan (y, x, (mpfr_rnd_t) r);
       mpfr_const_pi (x, MPFR_INVERT_RND((mpfr_rnd_t) r));
       mpfr_neg (x, x, (mpfr_rnd_t) r);
-      mpfr_div_2exp (x, x, 1, (mpfr_rnd_t) r);
+      mpfr_div_2ui (x, x, 1, (mpfr_rnd_t) r);
       if (mpfr_cmp (x, y))
         {
           printf ("Error: mpfr_atan(-Inf), rnd=%s\n",
@@ -443,7 +442,7 @@ smallvals_atan2 (void)
   mpfr_set_prec (y, 8);
   mpfr_set_prec (a, 8);
   old_emin = mpfr_get_emin ();
-  mpfr_set_emin (MPFR_EMIN_MIN);
+  set_emin (MPFR_EMIN_MIN);
 
   mpfr_set_si (y, 3, MPFR_RNDN);
   mpfr_set_exp (y, mpfr_get_emin ());
@@ -473,7 +472,7 @@ smallvals_atan2 (void)
   mpfr_atan2 (a, y, x, MPFR_RNDN);
   MPFR_ASSERTN (mpfr_zero_p (a));
 
-  mpfr_set_emin (old_emin);
+  set_emin (old_emin);
 
   mpfr_clears (a, x, y, (mpfr_ptr) 0);
 }
