@@ -2,8 +2,8 @@
    mpfr_lessequal_p, mpfr_lessgreater_p, mpfr_equal_p, mpfr_unordered_p
    functions.
 
-Copyright 2003, 2006-2017 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 2003, 2006-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -18,9 +18,8 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
 #include "mpfr-test.h"
 
@@ -43,7 +42,7 @@ cmp_tests (void)
       mpfr_set_prec (y, precy);
       mpfr_urandomb (x, RANDS);
       mpfr_urandomb (y, RANDS);
-      signx = randlimb () & 1;
+      signx = RAND_BOOL ();
       signy = randlimb () % 256 ? signx : 1 - signx;
       /* signy = signx most of the time (most interesting case) */
       if (signx)
@@ -100,7 +99,7 @@ eq_tests (void)
       mpfr_set_prec (x, precx);
       mpfr_set_prec (y, precx + (randlimb () % 64));
       mpfr_urandomb (x, RANDS);
-      if (randlimb () & 1)
+      if (RAND_BOOL ())
         mpfr_neg (x, x, MPFR_RNDN);
       mpfr_set (y, x, MPFR_RNDN);  /* exact -> x = y */
       if (mpfr_greater_p (x, y) || !mpfr_greaterequal_p (x, y) ||
