@@ -1,7 +1,7 @@
 /* Test file for mpfr_cot.
 
-Copyright 2005-2022 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 2005-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,9 +16,8 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
 #include "mpfr-test.h"
 
@@ -88,11 +87,8 @@ two2emin (mpfr_exp_t e)
   old_emin = mpfr_get_emin ();
   old_emax = mpfr_get_emax ();
 
-  if (mpfr_set_emin (-e) || mpfr_set_emax (e))
-    {
-      printf ("Can't change exponent range\n");
-      exit (1);
-    }
+  set_emin (-e);
+  set_emax (e);
 
   mpfr_inits2 (53, x, y, (mpfr_ptr) 0);
   for (i = -4; i <= 4; i++)
@@ -120,8 +116,8 @@ two2emin (mpfr_exp_t e)
       }
   mpfr_clears (x, y, (mpfr_ptr) 0);
 
-  mpfr_set_emin (old_emin);
-  mpfr_set_emax (old_emax);
+  set_emin (old_emin);
+  set_emax (old_emax);
 }
 
 int
