@@ -1,7 +1,7 @@
 /* Test file for mpfr_fma.
 
-Copyright 2001-2022 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 2001-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,9 +16,8 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
 #include "mpfr-test.h"
 
@@ -1035,7 +1034,7 @@ coverage (void)
 
   /* same example, but with overflow */
   emax = mpfr_get_emax ();
-  mpfr_set_emax (GMP_NUMB_BITS + 1);
+  set_emax (GMP_NUMB_BITS + 1);
   mpfr_set_ui_2exp (z, 1, mpfr_get_emax () - 1, MPFR_RNDZ);
   mpfr_clear_flags ();
   inex = mpfr_fma (s, x, y, z, MPFR_RNDN);
@@ -1045,7 +1044,7 @@ coverage (void)
   MPFR_ASSERTN(inex > 0);
   MPFR_ASSERTN(mpfr_inf_p (s) && mpfr_sgn (s) > 0);
   MPFR_ASSERTN(mpfr_overflow_p ());
-  mpfr_set_emax (emax);
+  set_emax (emax);
 
   mpfr_clear (x);
   mpfr_clear (y);
@@ -1293,11 +1292,11 @@ main (int argc, char *argv[])
             mpfr_urandomb (y, RANDS);
             mpfr_urandomb (z, RANDS);
 
-            if (randlimb () % 2)
+            if (RAND_BOOL ())
               mpfr_neg (x, x, MPFR_RNDN);
-            if (randlimb () % 2)
+            if (RAND_BOOL ())
               mpfr_neg (y, y, MPFR_RNDN);
-            if (randlimb () % 2)
+            if (RAND_BOOL ())
               mpfr_neg (z, z, MPFR_RNDN);
 
             rnd = RND_RAND_NO_RNDF ();
