@@ -1,7 +1,7 @@
 /* Test file for mpfr_nextabove, mpfr_nextbelow, mpfr_nexttoward.
 
-Copyright 2003-2022 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 2003-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,9 +16,8 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
 #include "mpfr-test.h"
 
@@ -60,11 +59,11 @@ generic_abovebelow (void)
       do
         mpfr_urandomb (x, RANDS);
       while (mpfr_cmp_ui (x, 0) == 0);
-      neg = randlimb () & 1;
+      neg = RAND_BOOL ();
       if (neg)
         mpfr_neg (x, x, MPFR_RNDN);
       mpfr_set (y, x, MPFR_RNDN);
-      below = randlimb () & 1;
+      below = RAND_BOOL ();
       if (below)
         mpfr_nextbelow (y);
       else
