@@ -1,7 +1,7 @@
 /* Test file for mpfr_round_nearest_away.
 
-Copyright 2012-2022 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 2012-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,9 +16,8 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
 #include "mpfr-test.h"
 
@@ -152,13 +151,13 @@ main (int argc, char *argv[])
 
   /* mpfr_round_nearest_away requires emin is not the smallest possible */
   if ((emin = mpfr_get_emin ()) == mpfr_get_emin_min ())
-    mpfr_set_emin (mpfr_get_emin_min () + 1);
+    set_emin (mpfr_get_emin_min () + 1);
 
   test_special ();
 
   test_nonspecial ();
 
-  mpfr_set_emin (emin);
+  set_emin (emin);
 
   tests_end_mpfr ();
   return 0;
