@@ -1,7 +1,7 @@
 /* Test file for mpfr_set_d and mpfr_get_d.
 
-Copyright 1999-2020 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 1999-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,9 +16,8 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
 #include <float.h>
 
@@ -193,18 +192,14 @@ main (int argc, char *argv[])
       exit (1);
     }
 
-  n = (argc==1) ? 500000 : atoi(argv[1]);
+  n = argc == 1 ? 500000 : atoi (argv[1]);
   for (k = 1; k <= n; k++)
     {
       do
         {
           d = DBL_RAND ();
         }
-#ifdef HAVE_SUBNORM_DBL
-      while (0);
-#else
       while (ABS(d) < DBL_MIN);
-#endif
       mpfr_set_d (x, d, (mpfr_rnd_t) 0);
       dd = mpfr_get_d1 (x);
       if (d != dd && !(Isnan(d) && Isnan(dd)))
