@@ -1,7 +1,7 @@
 /* Test file for multiple mpfr.h inclusion and intmax_t related functions
 
-Copyright 2010-2019 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 2010-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,9 +16,12 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
+
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #if HAVE_STDINT_H
 
@@ -52,6 +55,18 @@ https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
  * Note: If _MPFR_EXP_FORMAT == 4 (which is never the case by default),
  * a part of the above check is not done because <stdint.h> is included
  * before the first mpfr.h inclusion (see above).
+ *
+ * Moreover, assuming that this test is run on a platform that has
+ * <stdint.h> (most platforms do nowadays), without mini-gmp, this
+ * test also allows one to detect that mpfr.h can be included without
+ * any other inclusion before[*] (such as <stdio.h>). For instance,
+ * it can detect any unprotected use of FILE in the mpfr.h header
+ * file.
+ * [*] possibly except config.h when used, which is normally not the
+ *     case with a normal build. Anyway, if we decided to change that,
+ *     this inclusion would not change anything as config.h would only
+ *     have defines (such as HAVE_STDINT_H) currently provided as "-D"
+ *     compiler arguments.
  */
 #include <mpfr.h>
 
@@ -75,8 +90,8 @@ main (void)
   mpfr_clear (x);
   if (j != 1)
     {
-#ifdef MPFR_PRINTF_MAXLM
-      printf ("Error: got %" MPFR_PRINTF_MAXLM "d instead of 1.\n", j);
+#ifndef NPRINTF_J
+      printf ("Error: got %jd instead of 1.\n", j);
 #else
       printf ("Error: did not get 1.\n");
 #endif
