@@ -1,7 +1,7 @@
 /* Test file for mpfr_ui_pow and mpfr_ui_pow_ui.
 
-Copyright 2001-2022 Free Software Foundation, Inc.
-Contributed by the AriC and Caramba projects, INRIA.
+Copyright 2001-2025 Free Software Foundation, Inc.
+Contributed by the Pascaline and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -16,9 +16,8 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
-51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.
+If not, see <https://www.gnu.org/licenses/>. */
 
 #include "mpfr-test.h"
 
@@ -142,54 +141,49 @@ check1 (mpfr_ptr x, mpfr_prec_t prec, unsigned long nt, mpfr_rnd_t rnd)
   mpfr_clear (t);
 }
 
-int
-main (int argc, char *argv[])
+static void
+huge (void)
 {
-  mpfr_t x, y;
-  unsigned long int n;
+  mpfr_exp_t old_emin, old_emax;
+  mpfr_t x;
 
-  tests_start_mpfr ();
+  old_emin = mpfr_get_emin ();
+  old_emax = mpfr_get_emax ();
 
-  mpfr_init (x);
-  mpfr_init (y);
+  set_emin (MPFR_EMIN_MIN);
+  set_emax (MPFR_EMAX_MAX);
 
-  do n = randlimb (); while (n <= 1);
+  mpfr_init2 (x, 8);
 
-  MPFR_SET_INF(x);
-  mpfr_ui_pow (y, n, x, MPFR_RNDN);
-  if(!MPFR_IS_INF(y))
-    {
-      printf ("evaluation of function at INF does not return INF\n");
-      exit (1);
-    }
+  /* The purpose of this test is more to check that mpfr_ui_pow_ui
+     terminates (without taking much memory) rather than checking
+     the value of x. On 2023-02-13, the +Inf case was not handled
+     in the Ziv iteration, yielding an infinite loop, affecting
+     mpfr_log10 in particular. See
+       commit 90de094f0d9c309daca707aa227470d810866616
+  */
+  mpfr_ui_pow_ui (x, 5, ULONG_MAX, MPFR_RNDN);
+  if (MPFR_EMAX_MAX <= ULONG_MAX)  /* true with default _MPFR_EXP_FORMAT */
+    MPFR_ASSERTN (MPFR_IS_INF (x));
 
-  MPFR_CHANGE_SIGN(x);
-  mpfr_ui_pow (y, n, x, MPFR_RNDN);
-  if(!MPFR_IS_ZERO(y))
-    {
-      printf ("evaluation of function in -INF does not return 0");
-      exit (1);
-    }
+  mpfr_clear (x);
 
-  MPFR_SET_NAN(x);
-  mpfr_ui_pow (y, n, x, MPFR_RNDN);
-  if(!MPFR_IS_NAN(y))
-    {
-      printf ("evaluation of function in NAN does not return NAN");
-      exit (1);
-    }
+  set_emin (old_emin);
+  set_emax (old_emax);
+}
 
-  test1 ();
-
-  {
-  mpfr_t z, t;
+static void
+test2 (void)
+{
+  mpfr_t x, y, z, t;
   mpfr_prec_t prec;
   mpfr_rnd_t rnd;
   unsigned int n;
+  mpfr_prec_t p0 = 2, p1 = 100;
+  unsigned int N = 20;
 
-  mpfr_prec_t p0=2, p1=100;
-  unsigned int N=20;
-
+  mpfr_init2 (x, 2);
+  mpfr_init2 (y, 2);
   mpfr_init2 (z, 38);
   mpfr_init2 (t, 6);
 
@@ -197,38 +191,30 @@ main (int argc, char *argv[])
   mpfr_set_str_binary (t, "0.110000E5");
   mpfr_ui_pow (z, 3, t, MPFR_RNDN);
 
-  mpfr_set_prec (x, 2);
-  mpfr_set_prec (y, 2);
   mpfr_set_str (x, "-0.5", 10, MPFR_RNDZ);
   mpfr_ui_pow (y, 4, x, MPFR_RNDD);
-  if (mpfr_cmp_ui_2exp(y, 1, -1))
+  if (mpfr_cmp_ui_2exp (y, 1, -1))
     {
-      fprintf (stderr, "Error for 4^(-0.5), prec=2, MPFR_RNDD\n");
-      fprintf (stderr, "expected 0.5, got ");
-      mpfr_out_str (stderr, 2, 0, y, MPFR_RNDN);
-      fprintf (stderr, "\n");
+      printf ("Error for 4^(-0.5), prec=2, MPFR_RNDD\n");
+      printf ("expected 0.5, got ");
+      mpfr_out_str (stdout, 2, 0, y, MPFR_RNDN);
+      printf ("\n");
       exit (1);
     }
 
   /* problem found by Kevin on spe175.testdrive.compaq.com
      (03 Sep 2003), ia64 under HP-UX */
-  mpfr_set_prec (x, 2);
-  mpfr_set_prec (y, 2);
   mpfr_set_str (x, "0.5", 10, MPFR_RNDN);
   mpfr_ui_pow (y, 398441521, x, MPFR_RNDN);
-  if (mpfr_cmp_ui_2exp(y, 1, 14))
+  if (mpfr_cmp_ui_2exp (y, 1, 14))
     {
-      fprintf (stderr, "Error for 398441521^(0.5), prec=2, MPFR_RNDN\n");
-      fprintf (stderr, "expected 1.0e14, got ");
-      mpfr_out_str (stderr, 2, 0, y, MPFR_RNDN);
-      fprintf (stderr, "\n");
+      printf ("Error for 398441521^(0.5), prec=2, MPFR_RNDN\n");
+      printf ("expected 1.0e14, got ");
+      mpfr_out_str (stdout, 2, 0, y, MPFR_RNDN);
+      printf ("\n");
       exit (1);
     }
 
-  mpfr_clear (z);
-  mpfr_clear (t);
-
-  mpfr_set_prec (x, 2);
   mpfr_set_str (x, "0.5", 10, MPFR_RNDN);
   check1 (x, 2, 398441521, MPFR_RNDN);  /* 398441521 = 19961^2 */
 
@@ -245,11 +231,70 @@ main (int argc, char *argv[])
           check1 (x, prec, nt, rnd);
         }
     }
-  }
+
+  mpfr_clears (x, y, z, t, (mpfr_ptr) 0);
+}
+
+/* reverse the arguments n and x for tgeneric_ui.c */
+static int
+ui_pow_rev (mpfr_ptr y, mpfr_srcptr x, unsigned long n, mpfr_rnd_t rnd)
+{
+  return mpfr_ui_pow (y, n, x, rnd);
+}
+
+#define TEST_FUNCTION ui_pow_rev
+#define INTEGER_TYPE  unsigned long
+#define RAND_FUNCTION(x) mpfr_random2(x, MPFR_LIMB_SIZE (x), 1, RANDS)
+#define INT_RAND_FUNCTION() \
+  (randlimb () % 16 == 0 ? randulong () : (unsigned long) (randlimb () % 32))
+#include "tgeneric_ui.c"
+
+int
+main (int argc, char *argv[])
+{
+  mpfr_t x, y;
+  unsigned long int n;
+
+  tests_start_mpfr ();
+
+  mpfr_init (x);
+  mpfr_init (y);
+
+  do n = randlimb (); while (n <= 1);
+
+  MPFR_SET_INF (x);
+  mpfr_ui_pow (y, n, x, MPFR_RNDN);
+  if (! MPFR_IS_INF (y))
+    {
+      printf ("evaluation of function at INF does not return INF\n");
+      exit (1);
+    }
+
+  MPFR_CHANGE_SIGN (x);
+  mpfr_ui_pow (y, n, x, MPFR_RNDN);
+  if (! MPFR_IS_ZERO (y))
+    {
+      printf ("evaluation of function in -INF does not return 0\n");
+      exit (1);
+    }
+
+  MPFR_SET_NAN (x);
+  mpfr_ui_pow (y, n, x, MPFR_RNDN);
+  if (! MPFR_IS_NAN (y))
+    {
+      printf ("evaluation of function in NAN does not return NAN\n");
+      exit (1);
+    }
 
   mpfr_clear (x);
   mpfr_clear (y);
 
+  test1 ();
+  test2 ();
+  huge ();
+
+  test_generic_ui (MPFR_PREC_MIN, 100, 100);
+
   tests_end_mpfr ();
   return 0;
 }
